rm(list = ls())
set.seed(12345)

library(compositions)
library(rstan)
library(MASS)
library(coda)
library(xtable)
library(RColorBrewer)

source("depthfunctions.R")

runstan <- FALSE #fit the model using Rstan
processstan <- FALSE #process the stan output and save
makemodelplot <- FALSE #plot fitted model?
makeintegralplot <- FALSE #plot mean composition integrated over depth?
makederivativeplot <- FALSE #plot norm of derivative of composition wrt depth?
makedistanceplots <- FALSE #plot figures associated with dissimilarity between communities at different depths?
printestimates <- FALSE #print parameter estimates to LaTeX files?
makesimulateddata <- FALSE #generate simulated data sets?
processsimulateddata <- FALSE #collect results from multiple simulated data sets
summarizesimulateddata <- FALSE #summarize results from multiple simulated data sets
makedepthplot <- FALSE #kernel density plot of distribution of sampled depths

standataname <- "../../stanoutput/9taxadepthcholesky.RData" #will save rstan output here
processstanname <- "../../stanoutput/9taxaprocessed.RData" #will save slow stuff here
rstan_options(auto_write = TRUE)
options(mc.cores = parallel::detectCores())

rawdata <- read.table(file = "../data/alltransects.txt", header = TRUE, sep = "\t")
alldata <- data.frame(Bare.wall = rawdata$Bare.wall, 
				Bugula = rawdata$Bugula, 
				Solitary.ascidians = rawdata$Ciona.intestinalis + rawdata$Styela.clava,
				Algae = rawdata$Green.algae + rawdata$Red.algae,
				Sponges = rawdata$Halichondria + rawdata$Other.sponges,
				Diadumene.cincta = rawdata$Metridium.senile, #correct misidentification				
				Mytilus.edulis = rawdata$Mytilus.edulis,
				Aurelia.aurita = rawdata$Aurelia.aurita,
				Colonial.ascidians = rawdata$Trididemnum.cereum + rawdata$Botryllus.sp, #correct misidentification: Botrylloides violaceus, not Trididemnum cereum
				depth = rawdata$depth,
				transect = rawdata$transect)
write.table(alldata, sep = "\t", file = "../data/correcteddata.txt", row.names = FALSE, col.names = TRUE) #aggregate taxa, exclude very rare taxa, correct some misidentifications. These are the data used for analysis.

ntaxa <- dim(alldata)[2] - 2
nstills <- dim(alldata)[1]

V <- ilrBase(D = ntaxa)
tVinv <- gettVinv(V)
depthscale <- scale(alldata$depth, center = TRUE, scale = TRUE)
polydepth <- poly(depthscale, degree = 2)

if(runstan){
    transect_dat <- list(ntaxa = ntaxa, 
                         nstills = nstills,
                         counts = alldata[, 1:ntaxa],
                         depth = polydepth[, 1],
                         squareddepth = polydepth[, 2],
                         tVinv = tVinv)
    niter <- 1e4 #number of stan iterations: used 1e4 in paper, but 1e3 to demonstrate code
    fit <- stan(file = '../stan/depthanalysis.stan', data = transect_dat, 
			iter = niter, chains = 4) 

    plot(rstan::traceplot(fit, pars = c("beta0", "beta1", "beta2", "LSigma"), inc_warmup = TRUE))

    print(summary(fit, pars = c("beta0", "beta1", "beta2", "LSigma"))$summary)
    print("Over all parameters:")
    print(summary(summary(fit)$summary[,9:10]))
    save(fit, file = standataname)
} else {
	load(standataname)
}

if(processstan){
	rho <- extract(fit, pars= "rho")[[1]]
	beta0 <- extract(fit, pars= "beta0")[[1]]
	beta1 <- extract(fit, pars= "beta1")[[1]]
	beta2 <- extract(fit, pars= "beta2")[[1]]
        LSigma <- extract(fit, pars = "LSigma")[[1]]
        
	alpha0 <- array(dim = dim(beta0))
	alpha1 <- array(dim = dim(beta0))
	alpha2 <- array(dim = dim(beta0))
	
	for(i in 1:dim(beta0)[1]){
    		coefs.not.orthogonal <- coeffrompoly(z = polydepth, x = depthscale, beta0 = beta0[i, ], beta1 = beta1[i, ], beta2 = beta2[i, ])
    		alpha0[i, ] <- coefs.not.orthogonal$alpha0
    		alpha1[i, ] <- coefs.not.orthogonal$alpha1
    		alpha2[i, ] <- coefs.not.orthogonal$alpha2
	}

	rhomean <- apply(rho, c(2, 3), "mean")
	beta0mean <- colMeans(beta0)
	beta1mean <- colMeans(beta1)
	beta2mean <- colMeans(beta2)
        LSigmamean <- apply(LSigma, c(2, 3), "mean")
	ndepth <- 100

	newdepth <- seq(from = min(alldata$depth), to = max(alldata$depth), length.out = ndepth)
	newdepthscale <- as.numeric(scale(newdepth, center = TRUE, scale = TRUE))
	depthseq <- predict(polydepth, newdepthscale)

	prho <- predictrho(depth1 = depthseq[, 1], depth2 = depthseq[, 2], beta0 = beta0mean, beta1 = beta1mean, beta2 = beta2mean, V = V)$rho
	iter <- dim(rho)[1]
	allrho <- array(dim = c(iter, ndepth, ntaxa))
	allx <- array(dim = c(iter, ndepth, ntaxa - 1))
	for (j in 1:iter){
    		prh <- predictrho(depth1 = depthseq[, 1], depth2 = depthseq[, 2], beta0 = beta0[j, ], beta1 = beta1[j, ], beta2 = beta2[j, ], V = V)
    		allrho[j, ,] <- prh$rho
   		allx[j, , ] <- prh$x
	}

	derivs <- array(dim = c(iter, ndepth, ntaxa))
        dnorms <- array(dim = c(iter, ndepth))
	for(j in 1:iter){
            dd <- depthderivative(beta1 = alpha1[j, ], beta2 = alpha2[j, ], z = newdepthscale)
            dnorms[j, ] <- apply(dd, 2, norm, type = "2")
            derivs[j, , ] <- ilrInv(t(dd), V = V)
	}

	ilrmeans <- array(dim = c(iter, ntaxa - 1))
	for(j in 1:iter){
    		ilrmeans[j, ] <- t(depthmean(beta0 = alpha0[j, ], beta1 = alpha1[j, ], beta2 = alpha2[j, ], deep = max(depthscale), shallow = min(depthscale)))
	}

	means <- ilrInv(ilrmeans, V = V)

	#postmean is posterior mean of compositional mean wrt depth
	postmean <- ilrInv(colMeans(ilrmeans), V = V)
	credible <- HPDinterval(as.mcmc(means))

        dda <- depthdistfast(allrho)
        
	save(list = c("rho", "beta0", "beta1", "beta2", "alpha0", "alpha1", "alpha2", "LSigma", "rhomean", "beta0mean", "beta1mean", "beta2mean", "LSigmamean",
			"ndepth", "newdepth", "newdepthscale", "depthseq", "prho", "iter", "allrho", "allx", "derivs", "dnorms", "ilrmeans", "means", "postmean", "credible", "dda"), file = processstanname)
} else {
	load(processstanname)
}

if(makemodelplot == TRUE){
    plotmodelwithinsets(fname = "../latex/modeloutput.pdf", ntaxa = ntaxa, iter = iter, alldata = alldata, newdepth = newdepth, prho = prho, allx = allx, useclr = TRUE)
}

if(makeintegralplot == TRUE){
    integralplot(fname = "../latex/integral.pdf", postmean = postmean, alldata = alldata, credible = credible)
}

if(makederivativeplot == TRUE){
    nlines = 2000
    derivativeplot(fname = "../latex/derivative.pdf", dnorms = dnorms, newdepth = newdepth, nlines = nlines)
}

if(makedistanceplots == TRUE){
    plotdepthdistarray(depthdistarray = dda$depthdistarray, newdepth = newdepth, fname = "../latex/distances.pdf")
    plotdepthdistarray(depthdistarray = dda$depthdistnobarewallarray, newdepth = newdepth, fname = "../latex/distancesnobarewall.pdf")
    ispowering(ilrInv(alpha1, V = V), ilrInv(alpha2, V = V), c(1, 4, 5), names = names(alldata)[1:9], nlines = 1000, fname = "../latex/ispowering.pdf")
}

if(printestimates == TRUE){
    printBeta(beta0 = alpha0, beta1 = alpha1, beta2 = alpha2, fname = "../latex/coefficients.tex")
    printLSigma(LSigma = LSigma, fname = "../latex/LSigma.tex")
}


if(makesimulateddata == TRUE){
    reps <- 100
    simfilebase <- "../../simulateddata/simulated_"
    npoints <- rep(100, nstills) #number of points per photo
    Sigmamean <- LSigmamean %*% t(LSigmamean)
    for(r in 1:reps){
        simfilename <- paste(simfilebase, r, ".txt", sep = "")
        simulateonerep(simfilename = simfilename, beta0 = beta0mean, beta1 = beta1mean, beta2 = beta2mean, polydepth = polydepth, Sigma = Sigmamean, npoints = npoints, ntaxa = ntaxa, V = V)
    }
}

if(processsimulateddata == TRUE){
    simfilepath <- "../../simulateddata/"
    simresultsname <- paste(simfilepath, "simresults.RData", sep = "")
    dumpfiles <- list.files(path = simfilepath, pattern = ".txt$")
    inHPDs <- array(dim = c(length(dumpfiles), 60))
    mynames <- c("b01", "b02", "b03", "b04", "b05", "b06", "b07", "b08", "b11", "b12", "b13", "b14", "b15", "b16", "b17", "b18", "b21", "b22", "b23", "b24", "b25", "b26", "b27", "b28", "LSigma11", "LSigma21", "LSigma31", "LSigma41", "LSigma51", "LSigma61", "LSigma71", "LSigma81", "LSigma22", "LSigma32", "LSigma42", "LSigma52", "LSigma62", "LSigma72", "LSigma82", "LSigma33", "LSigma43", "LSigma53", "LSigma63", "LSigma73", "LSigma83", "LSigma44", "LSigma54", "LSigma64", "LSigma74", "LSigma84", "LSigma55", "LSigma65", "LSigma75", "LSigma85", "LSigma66", "LSigma76", "LSigma86", "LSigma77", "LSigma87", "LSigma88")
    colnames(inHPDs) <- mynames
    densities <- vector("list", length = length(dumpfiles))
    truetheta <- c(beta0mean, beta1mean, beta2mean, LSigmamean[lower.tri(LSigmamean, diag = TRUE)])
    for(j in 1:length(dumpfiles)){
            i <- dumpfiles[j]
            ipattern <- paste(i, "[[:digit:]].csv", sep = "")
            csvfiles <- list.files(path = simfilepath, pattern = ipattern)
            print(csvfiles)
            idens <- getsimdens(csvfiles = paste(simfilepath, csvfiles, sep = ""), truetheta = truetheta)
            inHPDs[j, ] <- idens$inHPDs
            densities[[j]] <- idens$densities
    }
    save(list = c("truetheta", "inHPDs", "densities", "mynames"), file = simresultsname)
}

if(summarizesimulateddata == TRUE){
    simfilepath <- "../../simulateddata/"
    simresultsname <- paste(simfilepath, "simresults.RData", sep = "")
    load(simresultsname)
    printinHPDs(inHPDs = inHPDs , betafname = "../latex/betainHPDs.tex", LSigmafname = "../latex/LSigmainHPDs.tex", s = ntaxa - 1)
    pdf(file = "../latex/simresults_beta.pdf")
    plotsimdens(truetheta = truetheta, densities = densities, labs = NA, parrange = c(1, 24), panels = c(8, 3))
    dev.off()
    pdf(file = "../latex/simresults_LSigma.pdf")
    plotsimdensmatrix(truetheta = truetheta, densities = densities, parrange = c(25, 60), s = 8)
    dev.off()
    
}

if(makedepthplot == TRUE){
    plotdepths(depths = alldata$depth, figname = "../latex/depthdensity.pdf")
}
