#Get predicted values of composition and logratio composition at a range of depths
#Arguments:
#depth1 and depth2 are first-order and second order components of orthogonal polynomial for
#depth of degree 2 
#beta0, beta1, beta2 are vectors of coefficients for intercept, depth and depth2
#V is ilr basis matrix from ilrBase()
#Value: a list as follows
#rho is array of predicted rho values (iterations x depths x taxa)
#x is array of predicted values on logratio scale (iterations x depths x (taxa - 1)
predictrho <- function(depth1, depth2, beta0, beta1, beta2, V){
	ndepth <- length(depth1) 
	x <- array(dim = c(ndepth, length(beta0)))
	for(i in 1: ndepth){
		x[i,] <- beta0 + beta1 * depth1[i] + beta2 * depth2[i]
	}
	rho <- ilrInv(x, V)
	return(list(rho = rho, x = x))
}

#finds the derivative of transformed community composition wrt depth
#Arguments:
#beta1 and beta2 are vectors of coefficients of depth and depth2
#z is depth (vector or scalar)
#value: vector of derivatives of each component
depthderivative <- function(beta1, beta2, z){
	return (beta1 + 2 * outer(beta2, z)) 
}

#finds the mean value of transformed community composition wrt depth
#Arguments:
#beta0, beta1 and beta2 are vectors of intercept, coefficients of depth and depth2
#deep is the deepest depth (upper limit of integration)
#shallow is the shallowest depth (lower limit of integration)
#value: vector of means of each transformed component
depthmean <- function(beta0, beta1, beta2, deep, shallow){
	return(1 / (deep - shallow) * ((beta0 * deep + beta1 * deep^2 / 2 + beta2 * deep^3 / 3) - (beta0 * shallow + beta1 * shallow^2 / 2 + beta2 * shallow^3 / 3)))
}

#get coefficients of ordinary quadratic regression from coefficients of regression using orthogonal polynomials
#Arguments:
#z is orthogonal polynomials (order 2) for explanatory variable x
#x is original explanatory variable
#beta0 is intercept vector from model using z
#beta1 is coefficient vector for z
#beta2 is coefficient vector for z^2
#Value:
#A list containing coefficient vectors alpha0, alpha1, alpha2 that we would get if we used x and x^2 as explanatory variables
#Source: pages 342-345 in Kennedy, W.J. Jr. and Gentle, J.E. (1980). Statistical Computing. Marcel Dekker, New York (and https://stackoverflow.com/questions/39031172/how-poly-generates-orthogonal-polynomials-how-to-understand-the-coefs-ret for an explanation of what poly() returns)
coeffrompoly <- function(z, x, beta0, beta1, beta2){
    xx <- scale(x, center = TRUE, scale = FALSE)
    n <- length(x)
    alpha <- attributes(z)$coefs$alpha
    norm2 <- attributes(z)$coefs$norm2
    rootl0 <- sqrt(n)
    rootl1 <- sqrt(sum(xx^2))
    xbar <- alpha[1]
    alphas <- alpha - xbar
    betas <- norm2[-1] / norm2[-length(norm2)]
    P0 <- 1
    P1 <- xx
    P2 <-  (xx - alphas[2]) * P1 - betas[2] * P0

    P0 <- P0 / rootl0
    P1 <- P1 / rootl1
    rootl2 <- sqrt(sum(P2^2))
    P2 <- P2 / rootl2

    delta00 <- 1
    delta01 <- 0
    delta02 <- 0
    delta10 <- -xbar / rootl1
    delta11 <- 1 / rootl1
    delta12 <- 0
    delta22 <- 1 / rootl2
    delta21 <- (-2 * xbar - alphas[2]) / rootl2
    delta20 <- (xbar^2 + alphas[2] * xbar - betas[2]) / rootl2
    delta <- matrix(c(delta00, delta01, delta02, delta10, delta11, delta12, delta20, delta21, delta22), nrow = 3, ncol = 3)
    B <- rbind(beta0, beta1, beta2)
    A <- delta %*% B
    return(list(alpha0 = A[1, ], alpha1 = A[2, ], alpha2 = A[3, ]))
}


#get Aitchison distance between predicted compositions at two depths
#Arguments:
#z1 and z2 are two scalar depths (should be centred and scaled)
#beta0, beta1, beta2 are coefficients (not orthogonal) for intercept, depth, squared depth, in ilr coordinates
#V is basis matrix for ilr transformation
#Value:
#Aitchison distance between predicted compositions
#(list: dist.all is with all parts, dist.no.bare.wall is without bare wall)
#Example: the folllowing are the same.
#depthdist(z1 = newdepthscale[1], z2 = newdepthscale[2], beta0 = alpha0[1, ], beta1 = alpha1[1, ], beta2 = alpha2[1, ], V = V)
#dist(ilr(allrho[1, 1:2, ]), method = "euclidean")
#dist(acomp(allrho[1, 1:2, ]))
depthdist <- function(z1, z2, beta0, beta1, beta2, V){
    gamma0 <- ilrInv(beta0, V)
    gamma1 <- ilrInv(beta1, V)
    gamma2 <- ilrInv(beta2, V)
    cdiff <- (z1 - z2) * gamma1 + (z1^2 - z2^2) * gamma2
    s <- length(gamma1)
    identity <- acomp(rep(1, s))
    mydata <- acomp(rbind(cdiff, identity))
    mydata.no.bare.wall <- acomp(rbind(cdiff[-1], rep(1, s - 1)))
    dist.all <- dist(mydata)
    dist.no.bare.wall <- dist(mydata.no.bare.wall)
    return(list(dist.all = dist.all, dist.no.bare.wall = dist.no.bare.wall))

    #the stuff below is equivalent
    #c1 <- gamma0 + z1 * gamma1 + z1^2 * gamma2
    #c2 <- gamma0 + z2 * gamma1 + z2^2 * gamma2
    #c <- acomp(rbind(c1, c2))
    #return(dist(c))
}

#get array of Aitchison distances between compositions at different depths.
#This way is faster than getdepthdistarray.
#Arguments:
#allrho is array of predicted compositions (iterations x depths x parts)
#Value:
#depthdistarray: an array of Aitchison distances, iterations x ndepth x ndepth, where ndepth is length of newdepthscale
#depthdistnobarewallarray: an array of Aitchison distances excluding bare wall, iterations x ndepth x ndepth, where ndepth is length of newdepthscale
#Example:
#dda <- depthdistfast(allrho = allrho)
depthdistfast <- function(allrho){
    niter <- dim(allrho)[1]
    ndepth <- dim(allrho)[2]
    depthdistarray <- array(dim = c(niter, ndepth, ndepth))
    depthdistnobarewallarray <- array(dim = c(niter, ndepth, ndepth))
    for(i in 1:niter){
        depthdistarray[i, , ] <- as.matrix(dist(acomp(allrho[i, , ]), diag = TRUE, upper = TRUE))
        depthdistnobarewallarray[i, , ] <- as.matrix(dist(acomp(allrho[i, , -1]), diag = TRUE, upper = TRUE))
    }
    return(list(depthdistarray = depthdistarray, depthdistnobarewallarray = depthdistnobarewallarray))    
}

#get array of Aitchison distances between compositions at different depths.
#This way is slow! Use depthdistfast() instead.
#Arguments:
#newdepthscale is vector of (centred and scaled) depths
#beta0, beta1, beta2 are intercept, depth and depth^2 coefficients (not orthogonal) in ilr coordinates (iterations x taxa)
#V is basis matrix for ilr transformation
#Value:
#depthdistarray: an array of Aitchison distances, iterations x ndepth x ndepth, where ndepth is length of newdepthscale
#depthdistnobarewallarray: an array of Aitchison distances excluding bare wall, iterations x ndepth x ndepth, where ndepth is length of newdepthscale
#Example (with a few iterations):
#dda <- getdepthdistarray(newdepthscale = newdepthscale, beta0 = alpha0[1:10, ], beta1 = alpha1[1:10, ], beta2 = alpha2[1:10, ], V = V)
getdepthdistarray <- function(newdepthscale, beta0, beta1, beta2, V = V){
    ndepth <- length(newdepthscale)
    niter <- dim(beta0)[1]
    depthdistarray <- array(dim = c(niter, ndepth, ndepth))
    depthdistnobarewallarray <- array(dim = c(niter, ndepth, ndepth))
    for(i in 1:niter){
        for(j in 1:ndepth){
            for(k in 1:ndepth){
                dd <- depthdist(z1 = newdepthscale[j], z2 = newdepthscale[k], beta0 = beta0[i, ], beta1 = beta1[i, ], beta2 = beta2[i, ], V = V)
                depthdistarray[i, j, k] <- dd$dist.all
                depthdistnobarewallarray[i, j, k] <- dd$dist.no.bare.wall
            }
        }
    }
    return(list(depthdistarray = depthdistarray, depthdistnobarewallarray = depthdistnobarewallarray))
}

#calculate width of 95% HPD interval for an array of distances
#Arguments:
#depthdistarray is a distance matrix for compositions at different depths (Monte Carlo iterations x depths x depths)
#Value: array (depths x depths) of widths of 95% HPD intervals
getHPDwidth <- function(depthdistarray){
    n <- dim(depthdistarray)[2]
    HPDwidth <- array(dim = c(n, n))
    for(i in 1:n){
        hpd <- HPDinterval(as.mcmc(depthdistarray[, i, ]))
        HPDwidth[i, ] <- hpd[, 2] - hpd[, 1]
    }
    return(HPDwidth)
}

#plot image of distance matrix wrt depth
#Arguments:
#depthdistarray: distance matrix (Monte Carlo iterations x depths x depths)
#newdepth: vector of depths at which we got expected composition
#fname: figure file name to produce
#Value:
#Image of mean distance matrix and standard deviation matrix
plotdepthdistarray <- function(depthdistarray, newdepth, fname){
    ddamean <- apply(depthdistarray, c(2, 3), "mean")
    ddahpdwidth <- getHPDwidth(depthdistarray)
    n <- dim(ddamean)[1]
    pdf(width = 14, height = 7, file = fname)
    par(mfrow = c(1, 2))
    par(mar = c(7, 4, 2, 4))
    par(mgp = c(2, 1, 0))
    par(pty = "s")
    par(xpd = NA)
    colorlevels <- gray.colors(100)
    image(newdepth, newdepth, ddamean, xlab = "depth / m", ylab = "depth / m", col = colorlevels, asp = 1, ylim = rev(range(newdepth)))
    text(0, 0, "(a) mean", pos = 4)
    image(newdepth, newdepth, ddahpdwidth, xlab = "depth / m", ylab = "depth / m", col = colorlevels, asp = 1, ylim = rev(range(newdepth)))
    text(0, 0, "(b) width of 95% HPD interval", pos = 4)
    par(pty = "m")
    colorBarpoly(colvals = colorlevels, zlim = range(ddamean), parfig = c(0.1, 0.4, 0.05, 0.075), digits = 0)
    colorBarpoly(colvals = colorlevels, zlim = range(ddahpdwidth), parfig = c(0.6, 0.9, 0.05, 0.075), digits = 1)
    dev.off()
}

#add color bar to image
#Arguments:
#colvals is colours required
#zlim is range of values represented by colours
#parfig is coordinates for color bar
#parmar is plot margins to use
#tlabs (optional) is tick labels
#tpos (optional) is tick label positions
#title (optional) is title for colour bar
#digits (default zero): number of digits on tick labels
#Value: adds color bar to existing image
colorBarpoly <- function(colvals, zlim, parfig = c(0.2, 0.8, 0.925, 0.95), parmar = c(0, 0, 0, 0), tlabs = NULL, tpos = NULL, title = NULL, digits = 0){
    par(fig = parfig, new = TRUE)
    ntick <- 2
    if(is.null(tpos)){
        tpos <- pretty(zlim, n = ntick)
        tlat <- (tpos - min(tpos)) / (diff(zlim))
    } else tlat <- tpos
    if(is.null(tlabs)){
        tlabs <- round(tpos, digits = digits)
    }        
    par(mar = parmar)
    image(matrix(1:length(colvals), ncol = 1), axes = FALSE, col = colvals)
    axis(1, at = tlat[-length(tlat)], tick = FALSE, labels = tlabs[-length(tlabs)], line = -0.5)
    if(!is.null(title)) text(0.5, 1, title)
    par(fig = c(0, 1, 0, 1), new = FALSE)
}

#produce a LaTeX table summarizing posterior distributions of parameters
#Arguments:
#beta0, beta1, beta2: arrays of parameters intercept, linear coefficient, quadratic coefficient (iterations x ilr coordinates): probably want non-orthogonal version of these
#fname: name of file to which LaTeX output should be written
#Value:
#LaTeX code for table of posterior means and 95% HPD intervals, written to fname
printBeta <- function(beta0, beta1, beta2, fname){
    beta0mean <- colMeans(beta0)
    beta0HPD <- HPDinterval(as.mcmc(beta0))
    beta1mean <- colMeans(beta1)
    beta1HPD <- HPDinterval(as.mcmc(beta1))
    beta2mean <- colMeans(beta2)
    beta2HPD <- HPDinterval(as.mcmc(beta2))
    s <- dim(beta0)[2]
    mytable <- array(dim = c(s, 3))
    for(i in 1:s){
        mytable[i, ] <- c(sprintf("%.2f (%.2f, %.2f)\n", beta0mean[i], beta0HPD[i, 1], beta0HPD[i, 2]), sprintf("%.2f (%.2f, %.2f)\n", beta1mean[i], beta1HPD[i, 1], beta1HPD[i, 2]), sprintf("%.2f (%.2f, %.2f)\n", beta2mean[i], beta2HPD[i, 1], beta2HPD[i, 2]))
    }
    print(xtable(mytable, type = "latex"), file = fname)
}

#produce a LaTeX table summarizing posterior distribution of Cholesky factor of covariance matrix
#Arguments:
#LSigma: cholesky factor of covariance matrix (iterations x components x components)
#fname: name of file to which LaTeX output should be written
#Value:
#LaTeX code for table of posterior means and 95% HPD intervals, written to fname
printLSigma <- function(LSigma, fname){
    s <- dim(LSigma)[2]
    LSigmatable <- array(0, dim = c(s, s))
    LSigmamean <- apply(LSigma, c(2, 3), "mean")
    for(j in 1:s){
        for(i in 1:(j-1)){
            LSigmatable[i, j] <- ""
        }
        for(i in j:s){
            myHPD <- HPDinterval(as.mcmc(LSigma[, i, j]))
            mymean <- sprintf(LSigmamean[i, j], fmt = "%.2f")
            LSigmatable[i, j] <- sprintf("%.2f (%.2f, %.2f)", LSigmamean[i, j], myHPD[1], myHPD[2])
        }
    }
    print(xtable(LSigmatable, type = "latex"), file = fname)

    
#    beta0mean <- colMeans(beta0)
#    beta0HPD <- HPDinterval(as.mcmc(beta0))
#    beta1mean <- colMeans(beta1)
#    beta1HPD <- HPDinterval(as.mcmc(beta1))
#    beta2mean <- colMeans(beta2)
#    beta2HPD <- HPDinterval(as.mcmc(beta2))
#    s <- dim(beta0)[2]
#    mytable <- array(dim = c(s, 3))
#    for(i in 1:s){
#        mytable[i, ] <- c(sprintf("%.2f (%.2f, %.2f)\n", beta0mean[i], beta0HPD[i, 1], beta0HPD[i, 2]), sprintf("%.2f (%.2f, %.2f)\n", beta1mean[i], beta1HPD[i, 1], beta1HPD[i, 2]), sprintf("%.2f (%.2f, %.2f)\n", beta2mean[i], beta2HPD[i, 1], beta2HPD[i, 2]))
#    }
#    print(xtable(mytable, type = "latex"), file = fname)
}


#graphical attempt to show whether gamma2 is a powering of gamma1
#Arguments:
#gamma1 is depth effect (as composition, iterations by parts)
#gamma2 is squared depth effect (as composition: iterations by parts)
#parts: a vector of three parts of the compositions to plot (e.g. c(1, 2, 3))
#names: the names of the parts of the composition (e.g. names(alldata)[1:9]
#nlines: number of lines (iterations) to plot for gamma1
#fname: file to which to save figure
#Value:
#A ternary plot with lines representing the set of powerings of gamma1 (for each of a set of iterations), and points representing gamma2 (for each of a set of iterations)
#Example:
#ispowering(ilrInv(alpha1, V = V), ilrInv(alpha2, V = V), c(1, 4, 5), names = names(alldata)[1:9], nlines = 1000, fname = "../latex/ispowering.pdf")
ispowering <- function(gamma1, gamma2, parts, names, nlines, fname){
    g1 <- acomp(gamma1[, parts])
    graphnames = sub("[.]", " ",names)
    names(g1) <- graphnames[parts]
    x <- seq(from = -100, to = 100, length.out = 100)
    myiter <- seq(from = 1, to = dim(g1)[1], length.out = nlines)
    pdf(file = fname, width = 7, height = 7 )
    plot(acomp(g1[1, ]), type = "n")
    for(i in myiter){
        lines(x * acomp(g1[i, ]), col = adjustcolor("black", alpha = 0.01))
    }
    plot(acomp(gamma2[myiter, parts]), add = TRUE, pch = 16, col = adjustcolor("black", alpha = 0.1))
    dev.off()
}

#plot fitted model
#Arguments:
#fname: name of pdf file in which to plot
#ntaxa: number of taxa
#iter: number of iterations
#alldata: data frame of observations. Rows are still images, variables are counts of taxa, then depth in metres and transect
#newdepth: vector of depths at which we have predicted compositon               #prho: predicted composition with posterior mean parameters (depths x taxa)
#allx: fitted logratio coordinates at depths in newdepth (iterations x depths x logratio coordinates)
#logscale (default FALSE) plot untransformed compositions if FALSE, otherwise logratio coordinates
#Value:
#pdf figure of fitted model, in file fname
plotmodel <- function(fname, ntaxa, iter, alldata, newdepth, prho, allx, logscale = FALSE){
    pdf(file = fname, height = 7, width = 7)
    italicize = c("FALSE", "TRUE", "FALSE", "FALSE", "FALSE", "TRUE", "TRUE", "TRUE", "FALSE") #italicize taxon name on y axis labels?
    if(logscale == FALSE){
        par(mfrow = c(3,3))
        par(mar = c(4, 4, 1, 1))
        for (i in 1:ntaxa){
            ylab = sub("[.]", " ",names(alldata)[1:9])[i]
            if(italicize[i] == TRUE) ylab <- bquote(italic(.(ylab)))
            plot(newdepth, prho[, i], type = "l", xlab = "Depth/m", ylab = ylab, ylim = c(0,1))
            mtext(paste0("(", letters[i], ")"), side = 3, adj = 0.05, line = -1.3)
            xHPD <- HPDinterval(as.mcmc(allrho[, , i]))
            polygon(c(rev(newdepth), newdepth), c(rev(xHPD[, 1]), xHPD[, 2]), col = adjustcolor("black", 0.4), border = NA)
            points (alldata$depth, alldata[, i ] / rowSums(alldata[, 1:ntaxa]), pch = 16, col = adjustcolor("black", 0.4))
        }
    } else {
        par(mfrow = c(3,3))
        for (i in 1:(ntaxa - 1)){
            plot(newdepth, allx[1, , 1], type = "n", xlim = range(newdepth), ylim = range(allx))
            for (j in seq(from = 1, to = iter, by = 100)) {
                lines(newdepth, allx[j, , i], col = adjustcolor("black", 0.01))    
            }
        }
    }
    dev.off()
}

#plot fitted model for nine taxa with inset panels
#Arguments:
#fname: name of pdf file in which to plot
#ntaxa: number of taxa
#iter: number of iterations
#alldata: data frame of observations. Rows are still images, variables are counts of taxa, then depth in metres and transect
#newdepth: vector of depths at which we have predicted compositon               #prho: predicted composition with posterior mean parameters (depths x taxa)
#allx: fitted logratio coordinates at depths in newdepth (iterations x depths x logratio coordinates)
#useclr: if TRUE (default), plot insets on clr scale (otherwise log scale)
                                        #Value:
#pdf figure of fitted model, in file fname
plotmodelwithinsets <- function(fname, ntaxa, iter, alldata, newdepth, prho, allx, useclr = TRUE){
    pdf(file = fname, height = 7, width = 7)
    italicize = c("FALSE", "TRUE", "FALSE", "FALSE", "FALSE", "TRUE", "TRUE", "TRUE", "FALSE") #italicize taxon name on y axis labels?
    layout(matrix(c(2, 1, 3, 4, 5, 6, 1, 1, 3, 3, 5, 5, 7, 8, 9, 10, 11, 12, 7, 7, 9, 9, 11, 11, 13, 14, 15, 16, 17, 18, 13, 13, 15, 15, 17, 17), nrow = 6, ncol = 6, byrow = TRUE))
    for (i in 1:ntaxa){
        par(mar = c(4, 3.5, 1, 1), tcl = -0.5, mgp = c(2, 1, 0))
        ylab = sub("[.]", " ",names(alldata)[1:9])[i]
        if(italicize[i] == TRUE) ylab <- bquote(italic(.(ylab)))
        plot(newdepth, prho[, i], type = "l", xlab = "Depth/m", ylab = ylab, ylim = c(0,1))
        if(i == 1){
            mtext(paste0("(", letters[i], ")"), side = 3, adj = 0.85, line = -1.3)
        } else {
            mtext(paste0("(", letters[i], ")"), side = 3, adj = 0.05, line = -1.3)
        }
        xHPD <- HPDinterval(as.mcmc(allrho[, , i]))
        polygon(c(rev(newdepth), newdepth), c(rev(xHPD[, 1]), xHPD[, 2]), col = adjustcolor("black", 0.4), border = NA)
        points (alldata$depth, alldata[, i ] / rowSums(alldata[, 1:ntaxa]), pch = 16, col = adjustcolor("black", 0.4))
        if(i ==1){
            par(mar = c(3, 4.5, 1.5, 0), tcl = -0.1, mgp = c(3, 0, 0))
        } else {
            par(mar = c(3, 3, 1.5, 1.5), tcl = -0.1, mgp = c(3, 0, 0))
        }
        if(useclr == TRUE){
            clrxHPD <- HPDinterval(as.mcmc(clr(allrho[, , i])))
            plot(newdepth, clr(prho[, i]), type = "l", xlab = "", ylab = "", ylim = range(clrxHPD), cex.axis = 0.5)
            polygon(c(rev(newdepth), newdepth), c(rev(clrxHPD[, 1]), clrxHPD[, 2]), col = adjustcolor("black", alpha.f = 0.4), border = NA)

        } else {
            plot(newdepth, prho[, i], type = "l", xlab = "", ylab = "", ylim = range(xHPD), cex.axis = 0.5, log = "y")
            polygon(c(rev(newdepth), newdepth), c(rev(xHPD[, 1]), xHPD[, 2]), col = adjustcolor("black", alpha.f = 0.4), border = NA)
        }
    }
    dev.off()
}


#italicize some of the elements of a vector of axis labels
#Arguments:
#labels: a character vector
#italicize: a logical vector
#Value:
#An expression in which the components for which italicize is TRUE are italic
make.italic <- function(labels, italicize){
     n <- length(labels)
     templabels <- vector("list", n)
     for(i in 1:n){
         if(italicize[i] == TRUE){
             templabels[[i]] <- bquote(italic(.(labels[i])))
         } else {
             templabels[[i]] <- labels[i]
         }
     }
     italiclabels <- as.expression(templabels)
     return(italiclabels)
}

#plot depth-integrated mean composition
#Arguments:
#fname: name of file in which to plot
#postmean: posterior mean of depth-integrated mean composition (vector, one element per taxon)
#alldata: data frame of observations. Rows are still images, variables are counts of taxa, then depth in metres and transect
#credible: 95% HPD interval for depth-integrated mean composition (array, taxa x 2)
#Value:
#pdf figure fname of depth-integrated mean composition
integralplot <- function(fname, postmean, alldata, credible){
    pdf(file = "../latex/integral.pdf", width = 7, height = 7)
    par(mar = c(10, 4, 1, 1))
    italicize <- c("FALSE", "TRUE", "FALSE", "FALSE", "FALSE", "TRUE", "TRUE", "TRUE", "FALSE")
    labels <- make.italic(labels = sub("[.]", " ", names(alldata)[1:9]), italicize = italicize)
    plot(as.vector(postmean), ylab = "Relative abundance", xaxt = "n", xlab = " ", ylim = c(0,1), pch = 16)
    axis(1, labels = labels, at=1:9, las = 3)
    for(i in 1:ntaxa){
        lines(c(i,i),credible[i,] )
    }
    dev.off()
}

#plot norm of derivative of composition wrt depth
#Arguments:
#fname: name of file in which to plot
#dnorms: array of norms of derivatives (iterations x depths)
#newdepth: vector of depths at which we have derivative
#nlines: how many of the individual lines from Monte Carlo iterations should we plot? Default 200.
#Value:
#pdf figure fname of norm of derivative of composition wrt depth
derivativeplot <- function(fname, dnorms, newdepth, nlines = 200){
    pdf(file = fname, width = 7, height = 7)
    xHPD <- HPDinterval(as.mcmc(dnorms))
    plot(range(newdepth), range(xHPD), type = "n", xlab = "Depth/m", ylab = "Norm of derivative of composition", cex.lab = 1.5)
    polygon(c(rev(newdepth), newdepth), c(rev(xHPD[, 1]), xHPD[, 2]), col = adjustcolor("black", 0.4), border = NA)
    for(i in 1:nlines){
        lines(newdepth, dnorms[i, ], col = adjustcolor("black", alpha.f = 0.05))
    }
    lines(newdepth, colMeans(dnorms), col = "white", lwd = 2)
    dev.off()
}

#generate a simulated data set
#Arguments:
#simfilename: name of file to which simulated data will be written
#beta0, beta1, beta2: intercept, depth and squared depth coefficient vectors in ilr coordinates (from orthogonal polynomials)
#polydepth: orthogonal polynomial for depth and squared depth
#Sigma: error covariance matrix
#npoints: vector of number of points counted in each photo
#ntaxa: number of taxa
#V: ilr basis matrix from ilrBase()
simulateonerep <- function(simfilename, beta0, beta1, beta2, polydepth, Sigma, npoints, ntaxa, V){
    nstills <- dim(polydepth)[1]
    epsilon <- mvrnorm(n = nstills, mu = rep(0, ntaxa - 1), Sigma = Sigma)
    B <- rbind(beta0, beta1, beta2)
    X <- cbind(1, polydepth)
    z <- X %*% B + epsilon
    rho <- ilrInv(z, V = V)
    counts <- array(dim = c(nstills, ntaxa))
    for(i in 1:nstills){
        counts[i, ] <- rmultinom(n = 1, size = npoints[i], prob = rho[i, ])
    }
    depth <- polydepth[, 1]
    squareddepth <- polydepth[, 2]
    tVinv <- gettVinv(V)
    stan_rdump(c("ntaxa", "nstills", "counts", "depth", "squareddepth", "tVinv"), file = simfilename, quiet = FALSE, append = FALSE)
}

#generalized inverse of transpose of V, needed to do inverse of ilr in stan
#Argument: V, used in ilr transformation
#Value: generalized inverse of transpose of V
#Note: I think there is a simpler way of getting this...
gettVinv <- function(V){
    return(ginv(t(V)))
}

#read data from a set of .csv Stan output files, extract parameters
#Arguments:
#csvfiles: character vector of file names
#truetheta is vector of true parameter values
#Value:
#returns a list containing:
#inHPDs: 1 x 60 array, with value TRUE if the corresponding true parameter is in the 95% HPD interval for the estimate. Ordered as in colnames(inHPDs) in processsimulateddata block, depthanalysis9taxa.R
#densities: 60-element list of kernel density estimates for each parameter in order as above, each the output of density()
getsimdens <- function(csvfiles, truetheta){
    gc()
    standata <- read_stan_csv(csvfiles = csvfiles)
    beta0 <- extract(standata, pars = "beta0")[[1]]
    beta1 <- extract(standata, pars = "beta1")[[1]]
    beta2 <- extract(standata, pars = "beta2")[[1]]
    LSigma <- extract(standata, pars = "LSigma")[[1]]
    rm(standata)
    gc()
    densities <- vector("list", length = length(truetheta))
    inHPDs <- array(dim = c(1, length(truetheta)))
    for(i in 1:8){
        densities[[i]] <- density(beta0[, i], bw = "SJ")
        inHPDs[i] <- inHPD(beta0[, i], truetheta[i])
    }
    for(i in 1:8){
        densities[[i + 8]] <- density(beta1[, i], bw = "SJ")
        inHPDs[i + 8] <- inHPD(beta1[, i], truetheta[i + 8])
    }
    for(i in 1:8){
        densities[[i + 16]] <- density(beta2[, i], bw = "SJ")
        inHPDs[i + 16] <- inHPD(beta2[, i], truetheta[i + 16])
    }
    k <- 1
    for(j in 1:8){
        for(i in j:8){
            densities[[k + 24]] <- density(LSigma[, i, j], bw = "SJ")
            inHPDs[k + 24] <- inHPD(LSigma[, i, j], truetheta[k + 24])
            k <- k + 1
        }
    }
    return(list(inHPDs = inHPDs, densities = densities))
    }

#is true value in 95% HPD interval?
#Arguments:
#theta is a vector of estimates of a parameter (iterations x 1)
#truetheta is true value
#Value:
#returns a logical value: is true value in 95% HPD interval?
inHPD <- function(theta, truetheta){
    hpdi <- HPDinterval(as.mcmc(theta))
    return(truetheta >= hpdi[1] & truetheta <= hpdi[2])
}

#plot collated output from simulations
#Arguments:
#truetheta is vector of true parameter values
#densities is a list of lists (first dimension replicates, second dimension parameters): each element is a kernel density estimate
#labs is vector of parameter names
#parrange: which parameters to plot? Two-element vector
#panels: two-element vector with number of rows and columns of panels to plot
#Value: overlaid kernel density plots for selected parameters, with true values
plotsimdens <- function(truetheta, densities, labs, parrange, panels){
    par(mfcol = panels)
    par(mar = c(2, 1, 0.5, 1))
    par(mgp = c(3, 0.1, 0))
    par(tcl = -0.1)
    ndens <- length(densities)
    for(i in seq(from = parrange[1], to = parrange[2])){
        plotparm(densities = densities, i = i, ndens = ndens, truetheta = truetheta, labs = labs)
    }
 }

#plot collated output from simulations: lower triangular matrix layout
#Arguments:
#truetheta is vector of true parameter values
#densities is a list of lists (first dimension replicates, second dimension parameters): each element is a kernel density estimate
#parrange: which parameters to plot? Two-element vector
#s: number of components
#Value: overlaid kernel density plots for selected parameters, with true values, in lower-triangular matrix layout
plotsimdensmatrix <- function(truetheta, densities, parrange, s){
    par(mfcol = c(s, s))
    par(mar = c(2, 1, 0.5, 1))
    par(mgp = c(3, 0.1, 0))
    par(tcl = -0.1)
    ndens <- length(densities)
    mycount <- parrange[1]
    for(j in 1:s){
         for(i in j:s){
            plotparm(densities = densities, i = mycount, ndens = ndens, truetheta = truetheta, labs = NA)
            mycount <- mycount + 1
         }
         if(j < s){
             for(i in 1:j){
                 plot.new()
             }
         }
    }
 }


#plot density estimates for a single parameter from multiple simulations
#Arguments:
#densities is a list of lists (first dimension replicates, second dimension parameters): each element is a kernel density estimate
#i is the element we want to plot (second dimension of densities)
#ndens is number of simulations
#truetheta is vector of true parameter values
#labs is vector of parameter names
#Value: plot with overlaid kernel density plots for each replicate
plotparm <- function(densities, i, ndens, truetheta, labs){
    xrange <- 0
    yrange <- 0
    mycolors <- brewer.pal(3, "Dark2")
    for(j in 1:ndens){
        xrange <- range(c(xrange, densities[[j]][[i]]$x))
        yrange <- range(c(yrange, densities[[j]][[i]]$y))
    }
    plot(densities[[1]][[i]], xlim = xrange, ylim = yrange, col = adjustcolor("black", 0.1), main = "", xlab = "", ylab = "", cex.axis = 0.5)
    for(j in 1:ndens){
      lines(densities[[j]][[i]], col = adjustcolor("black", 0.1))      
    }
    abline(v = truetheta[i], col = mycolors[1], lwd=2)
    parusr <- par("usr")
    text(parusr[1] + 0.1 * (parusr[2] - parusr[1]), parusr[4] - 0.15 * (parusr[4] - parusr[3]), labs[i], cex = 0.5)
}

#produce a LaTeX table summarizing proportion of replicate simulations in which credible interval contained true parameter values
#Arguments:
#inHPDs: 1 x 60 array, with value TRUE if the corresponding true parameter is in the 95% HPD interval for the estimate. Ordered as in colnames(inHPDs) in processsimulateddata block, depthanalysis9taxa.R
#betafname: name of file to which LaTeX output for beta coefficients should be written
#LSigmafname: name of file to which LaTeX output for LSigma should be written
#s: number of ilr components
#Value:
#LaTeX code for table of number of replicates in which 95% credible interval for each beta element included true value (written to betafname), and corresponding table for LSigma (written to LSigmafname)
printinHPDs <- function(inHPDs, betafname, LSigmafname, s){
    betatable <- array(dim = c(s, 3))
    LSigmatable <- array(0, dim = c(s, s))
    colsums <- colSums(inHPDs)
    mycount <- 3 * s + 1
    for(j in 1:s){
        betatable[j, 1] <- sprintf(colsums[j], fmt = "%d")
        betatable[j, 2] <- sprintf(colsums[j + s], fmt = "%d")
        betatable[j, 3] <- sprintf(colsums[j + 2 * s], fmt = "%d")
        for(i in j:s){
            LSigmatable[i, j] <- sprintf(colsums[mycount], fmt = "%d")
            mycount <- mycount + 1
        }
    }
    print(xtable(betatable, type = "latex"), file = betafname)
    print(xtable(LSigmatable, type = "latex"), file = LSigmafname)
}

#plot distribution of sampled depths
#Arguments:
#depths: vector of sampled depths
#figname: name of file to which figure should be written
#Value:
#kernel density plot of distribution of sampled depths, in pdf figure figname
plotdepths <- function(depths, figname){
    pdf(figname, width = 7, height = 7)
    depthdensity <- density(depths, from = 0, to = max(depths), bw = "SJ")
    plot(depthdensity, main = "", xlab = "Depth/m", ylab = "Density", ylim = c(0, max(depthdensity$y)))
    dev.off()
}
